"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var typescript_1 = require("typescript");
var logger_1 = require("../logger/logger");
var typescript_utils_1 = require("../util/typescript-utils");
function purgeTranspiledDecorators(filePath, originalFileContent, ionicAngularDir, angularDir, srcDir, magicString) {
    if (filePath.indexOf(angularDir) >= 0 || filePath.indexOf(ionicAngularDir) >= 0 || filePath.indexOf(srcDir) >= 0) {
        logger_1.Logger.debug("[decorators] purgeTranspiledDecorators: processing " + filePath + " ...");
        var typescriptFile = typescript_utils_1.getTypescriptSourceFile(filePath, originalFileContent);
        var expressionsToRemove = getTranspiledDecoratorExpressionStatements(typescriptFile);
        expressionsToRemove.forEach(function (expression) {
            magicString.overwrite(expression.pos, expression.end, '');
        });
        logger_1.Logger.debug("[decorators] purgeTranspiledDecorators: processing " + filePath + " ...");
    }
    return magicString;
}
exports.purgeTranspiledDecorators = purgeTranspiledDecorators;
function getTranspiledDecoratorExpressionStatements(sourceFile) {
    var expressionStatements = typescript_utils_1.findNodes(sourceFile, sourceFile, typescript_1.SyntaxKind.ExpressionStatement, false);
    var toReturn = [];
    expressionStatements.forEach(function (expressionStatement) {
        if (expressionStatement && expressionStatement.expression
            && expressionStatement.expression.kind === typescript_1.SyntaxKind.CallExpression
            && expressionStatement.expression.expression
            && expressionStatement.expression.expression.text === '___decorate') {
            toReturn.push(expressionStatement);
        }
        else if (expressionStatement && expressionStatement.expression
            && expressionStatement.expression.kind === typescript_1.SyntaxKind.BinaryExpression
            && expressionStatement.expression.right
            && expressionStatement.expression.right.kind === typescript_1.SyntaxKind.CallExpression
            && expressionStatement.expression.right.expression
            && expressionStatement.expression.right.expression.text === '___decorate') {
            expressionStatement.expression.right.arguments.forEach(function (argument) {
                if (argument.kind === typescript_1.SyntaxKind.ArrayLiteralExpression) {
                    var injectableFound = false;
                    for (var _i = 0, _a = argument.elements; _i < _a.length; _i++) {
                        var element = _a[_i];
                        if (element.kind === typescript_1.SyntaxKind.CallExpression && element.expression
                            && element.expression.text === 'Injectable') {
                            injectableFound = true;
                            break;
                        }
                    }
                    if (!injectableFound) {
                        toReturn.push(expressionStatement);
                    }
                }
            });
        }
    });
    return toReturn;
}
function purgeStaticFieldDecorators(filePath, originalFileContent, ionicAngularDir, angularDir, srcDir, magicString) {
    if (filePath.indexOf(angularDir) >= 0 || filePath.indexOf(ionicAngularDir) >= 0 || filePath.indexOf(srcDir) >= 0) {
        logger_1.Logger.debug("[decorators] purgeStaticFieldDecorators: processing " + filePath + " ...");
        var typescriptFile = typescript_utils_1.getTypescriptSourceFile(filePath, originalFileContent);
        var decoratorExpressionStatements = getDecoratorsExpressionStatements(typescriptFile);
        removeDecorators(decoratorExpressionStatements, magicString);
        var propDecoratorsExpressionStatements = getPropDecoratorsExpressionStatements(typescriptFile);
        removePropDecorators(propDecoratorsExpressionStatements, magicString);
        logger_1.Logger.debug("[decorators] purgeStaticFieldDecorators: processing " + filePath + " ... DONE");
    }
    return magicString;
}
exports.purgeStaticFieldDecorators = purgeStaticFieldDecorators;
function getDecoratorsExpressionStatements(typescriptFile) {
    var expressionStatements = typescript_utils_1.findNodes(typescriptFile, typescriptFile, typescript_1.SyntaxKind.ExpressionStatement, false);
    var decoratorExpressionStatements = [];
    for (var _i = 0, expressionStatements_1 = expressionStatements; _i < expressionStatements_1.length; _i++) {
        var expressionStatement = expressionStatements_1[_i];
        if (expressionStatement.expression && expressionStatement.expression.left && expressionStatement.expression.left.name && expressionStatement.expression.left.name.text === 'decorators') {
            decoratorExpressionStatements.push(expressionStatement);
        }
    }
    return decoratorExpressionStatements;
}
function getPropDecoratorsExpressionStatements(typescriptFile) {
    var expressionStatements = typescript_utils_1.findNodes(typescriptFile, typescriptFile, typescript_1.SyntaxKind.ExpressionStatement, false);
    var decoratorExpressionStatements = [];
    for (var _i = 0, expressionStatements_2 = expressionStatements; _i < expressionStatements_2.length; _i++) {
        var expressionStatement = expressionStatements_2[_i];
        if (expressionStatement.expression && expressionStatement.expression.left && expressionStatement.expression.left.name && expressionStatement.expression.left.name.text === 'propDecorators') {
            decoratorExpressionStatements.push(expressionStatement);
        }
    }
    return decoratorExpressionStatements;
}
function removeDecorators(decoratorExpressionStatements, magicString) {
    decoratorExpressionStatements.forEach(function (expressionStatement) {
        if (expressionStatement.expression && expressionStatement.expression.right && expressionStatement.expression.right.elements) {
            var numPotentialNodesToRemove = expressionStatement.expression.right.elements.length;
            var objectLiteralsToPurge_1 = [];
            expressionStatement.expression.right.elements.forEach(function (objectLiteral) {
                if (objectLiteral.properties && objectLiteral.properties.length > 1) {
                    if (objectLiteral.properties[0].name && objectLiteral.properties[0].name.text === 'type'
                        && canRemoveDecoratorNode(objectLiteral.properties[0].initializer.text)) {
                        // sweet, we can remove the object literal
                        objectLiteralsToPurge_1.push(objectLiteral);
                    }
                }
            });
            if (objectLiteralsToPurge_1.length === numPotentialNodesToRemove) {
                // we are removing all decorators, so just remove the entire expression node
                magicString.overwrite(expressionStatement.pos, expressionStatement.end, '');
            }
            else {
                // we are removing a subset of decorators, so remove the individual object literal findNodes
                objectLiteralsToPurge_1.forEach(function (objectLiteralToPurge) {
                    magicString.overwrite(objectLiteralToPurge.pos, objectLiteralToPurge.end, '');
                });
            }
        }
    });
}
function removePropDecorators(propDecoratorExpressionStatements, magicString) {
    propDecoratorExpressionStatements.forEach(function (expressionStatement) {
        magicString.overwrite(expressionStatement.pos, expressionStatement.end, '');
    });
}
function canRemoveDecoratorNode(decoratorType) {
    if (decoratorType === exports.COMPONENT) {
        return true;
    }
    else if (decoratorType === exports.CONTENT_CHILD_DECORATOR) {
        return true;
    }
    else if (decoratorType === exports.CONTENT_CHILDREN_DECORATOR) {
        return true;
    }
    else if (decoratorType === exports.DIRECTIVE_DECORATOR) {
        return true;
    }
    else if (decoratorType === exports.HOST_DECORATOR) {
        return true;
    }
    else if (decoratorType === exports.HOST_BINDING_DECORATOR) {
        return true;
    }
    else if (decoratorType === exports.HOST_LISTENER_DECORATOR) {
        return true;
    }
    else if (decoratorType === exports.INPUT_DECORATOR) {
        return true;
    }
    else if (decoratorType === exports.NG_MODULE_DECORATOR) {
        return true;
    }
    else if (decoratorType === exports.OUTPUT_DECORATOR) {
        return true;
    }
    else if (decoratorType === exports.PIPE_DECORATOR) {
        return true;
    }
    else if (decoratorType === exports.VIEW_CHILD_DECORATOR) {
        return true;
    }
    else if (decoratorType === exports.VIEW_CHILDREN_DECORATOR) {
        return true;
    }
    return false;
}
exports.COMPONENT = 'Component';
exports.CONTENT_CHILD_DECORATOR = 'ContentChild';
exports.CONTENT_CHILDREN_DECORATOR = 'ContentChildren';
exports.DIRECTIVE_DECORATOR = 'Directive';
exports.HOST_DECORATOR = 'Host';
exports.HOST_BINDING_DECORATOR = 'HostBinding';
exports.HOST_LISTENER_DECORATOR = 'HostListener';
exports.INPUT_DECORATOR = 'Input';
exports.NG_MODULE_DECORATOR = 'NgModule';
exports.OUTPUT_DECORATOR = 'Output';
exports.PIPE_DECORATOR = 'Pipe';
exports.VIEW_CHILD_DECORATOR = 'ViewChild';
exports.VIEW_CHILDREN_DECORATOR = 'ViewChildren';
